'use strict';
const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const { questions } = require('./js/questions');
const { answersConfig } = require('./js/answers.js');

module.exports = class extends Generator {
  prompting() {
    // Have Yeoman greet the user.
    this.log(
      yosay(
        `Welcome to the ultimate ${chalk.red('generator-spring-mybatis-crud-generator')} generator!`
      )
    );


    return this.prompt(questions).then(props => {
      // To access props later use this.props.someAnswer;
      this.crudConfig = answersConfig(props);
    });
  }

  writing() {
    const module = this.crudConfig.module.charAt(0).toUpperCase() + this.crudConfig.module.slice(1);
    const fileName = this.crudConfig.module.toLowerCase()
    //common
    //biz
    //dto
    this.destinationRoot("src/main/java/" + this.crudConfig.package_path);
    //common
    //advise
    this.fs.copyTpl(
      this.templatePath('./mongo/common/advise/ControllerAdviseException.java'),
      this.destinationPath('common/advise/ControllerAdviseException.java'),
      this.crudConfig
    )
    //config
    this.fs.copyTpl(
      this.templatePath('./mongo/common/config/SwaggerConfig.java'),
      this.destinationPath('common/config/SwaggerConfig.java'),
      this.crudConfig
    )
    //exception
    this.fs.copyTpl(
      this.templatePath('./mongo/common/exception/AbstractException.java'),
      this.destinationPath('common/exception/AbstractException.java'),
      this.crudConfig
    )
    this.fs.copyTpl(
      this.templatePath('./mongo/common/exception/BusinessException.java'),
      this.destinationPath('common/exception/BusinessException.java'),
      this.crudConfig
    )

    //type
    this.fs.copyTpl(
      this.templatePath('./mongo/common/constants/SysHttpResultCode.java'),
      this.destinationPath('common/constants/SysHttpResultCode.java'),
      this.crudConfig
    )

    //controller
    if(!this.fs.exists("common/controller/AbstractController.java"))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/controller/AbstractController.java'),
        this.destinationPath('common/controller/AbstractController.java'),
        this.crudConfig
      )
    }

    if(!this.fs.exists("common/controller/ResponseBuilderMessage.java"))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/controller/ResponseBuilderMessage.java'),
        this.destinationPath('common/controller/ResponseBuilderMessage.java'),
        this.crudConfig
      )
    }
    if(!this.fs.exists("common/controller/ResponseMessage.java"))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/controller/ResponseMessage.java'),
        this.destinationPath('common/controller/ResponseMessage.java'),
        this.crudConfig
      )
    }
    //request
    if(!this.fs.exists("common/request/PageableRequest.java"))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/request/PageableRequest.java'),
        this.destinationPath('common/request/PageableRequest.java'),
        this.crudConfig
      )
    }
  
   
    // //response
    if(!this.fs.exists("common/response/PageableResponse.java"))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/response/PageableResponse.java'),
        this.destinationPath('common/response/PageableResponse.java'),
        this.crudConfig
      )
    }
    
    if(!this.fs.exists('common/mapper/AbstractMapper.java'))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/mapper/AbstractMapper.java'),
        this.destinationPath('common/mapper/AbstractMapper.java'),
        this.crudConfig
      )
    }
     //utils
    if(!this.fs.exists('common/utils/PageableUtils.java'))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/utils/PageableUtils.java'),
        this.destinationPath('common/utils/PageableUtils.java'),
        this.crudConfig
      )
    }
    if(!this.fs.exists('common/utils/SearchUtils.java'))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/utils/SearchUtils.java'),
        this.destinationPath('common/utils/SearchUtils.java'),
        this.crudConfig
      )
    }
    if(!this.fs.exists('common/utils/FilterUtils.java'))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/utils/FilterUtils.java'),
        this.destinationPath('common/utils/FilterUtils.java'),
        this.crudConfig
      )
    }
    if(!this.fs.exists('common/utils/MongoEntityUtils.java'))
    {
      this.fs.copyTpl(
        this.templatePath('./mongo/common/utils/MongoEntityUtils.java'),
        this.destinationPath('common/utils/MongoEntityUtils.java'),
        this.crudConfig
      )
    }
    //build module path
    this.destinationRoot(fileName);

    this.fs.copyTpl(
      this.templatePath('./mongo/application/biz/dto/ApplicationDTO.java'),
      this.destinationPath("biz/dto/"+ module +"DTO.java"),
      this.crudConfig
    );
    this.fs.copyTpl(
      this.templatePath('./mongo/application/biz/dto/ApplicationSearchDTO.java'),
      this.destinationPath("biz/dto/"+ module +"SearchDTO.java"),
      this.crudConfig
    );
    //entity
    this.fs.copyTpl(
      this.templatePath('./mongo/application/biz/entity/ApplicationEntity.java'),
      this.destinationPath("biz/entity/"+ module +"Entity.java"),
      this.crudConfig
    );
     //mapper
     this.fs.copyTpl(
      this.templatePath('./mongo/application/biz/mapper/ApplicationMapper.java'),
      this.destinationPath("biz/mapper/"+ module +"Mapper.java"),
      this.crudConfig
    );
    // //service
    this.fs.copyTpl(
        this.templatePath('./mongo/application/biz/service/ApplicationService.java'),
        this.destinationPath("biz/service/"+ module +"Service.java"),
        this.crudConfig
      );
    // //serviceimpl
    this.fs.copyTpl(
          this.templatePath('./mongo/application/biz/service/impl/ApplicationServiceImpl.java'),
          this.destinationPath("biz/service/impl/"+ module +"ServiceImpl.java"),
          this.crudConfig
        );
    // //Web
    //  //controller
     this.fs.copyTpl(
      this.templatePath('./mongo/application/web/controller/ApplicationController.java'),
      this.destinationPath("web/controller/"+ module +"Controller.java"),
      this.crudConfig
    );
    //vo
    //request
       this.fs.copyTpl(
        this.templatePath('./mongo/application/web/vo/request/ApplicationCreatedRequest.java'),
        this.destinationPath("web/vo/request/"+ module +"CreatedRequest.java"),
        this.crudConfig
      );
      this.fs.copyTpl(
        this.templatePath('./mongo/application/web/vo/request/ApplicationUpdatedRequest.java'),
        this.destinationPath("web/vo/request/"+ module +"UpdatedRequest.java"),
        this.crudConfig
      );
      this.fs.copyTpl(
        this.templatePath('./mongo/application/web/vo/request/ApplicationPageableRequest.java'),
        this.destinationPath("web/vo/request/"+ module +"PageableRequest.java"),
        this.crudConfig
      );
    //response
      this.fs.copyTpl(
        this.templatePath('./mongo/application/web/vo/response/ApplicationResponse.java'),
        this.destinationPath("web/vo/response/"+ module +"Response.java"),
        this.crudConfig
      );
      this.fs.copyTpl(
        this.templatePath('./mongo/application/web/vo/response/ApplicationItemResponse.java'),
        this.destinationPath("web/vo/response/"+ module +"ItemResponse.java"),
        this.crudConfig
      );
  }
};